==> 0.0.3.tar.gz: lib/src/socket_user.dart <==
library super_sockets.socket_user;

import 'dart:io';
import 'dart:async';
import 'dart:convert';
import 'dart:collection';
import 'package:json_object/json_object.dart';
import 'stream_enhancer.dart';

class SocketUser {
  String _name;
  WebSocket _socket;
  Map<String, WriteableStreamEnhancer<JsonObject>> _router;
  Function _convert;
  List<List<int>> _blobList;
  JsonDecoder _decoder;
    
  Stream<JsonObject> messageStream(String type) {
    _ensureExistence(type);
    return _router[type].stream;
  }
  
  Future<JsonObject> receive(String type) {
    _ensureExistence(type);
    return _router[type].next();
  }
  
  void send(String type, JsonObject data){
    var sendable = new JsonObject();
    var blobQueue = new Queue<List<int>>();
    var fn = (value) {
      if (value is List<int>) {
        blobQueue.add(value);
        return "p:${blobQueue.length - 1}";
      } else if (value is String) {
        return ":$value";
      } else {
        return value;
      }
    };
    var jec = new JsonEncoder(fn);

    sendable.type = type;
    sendable.data = data;
    
    String jstring = jec.convert(sendable);
    for (List<int> blob in blobQueue) {
      _socket.add(blob);
    }
    
    _socket.add(jstring);
  }
  
  _ensureExistence(String type) {
    if (!_router.containsKey(type)) {
      _router[type] = new WriteableStreamEnhancer<JsonObject>.fromScratch();
    }
  }
    
  _socketListener(dynamic data) {
    if (data is List<int>) {
      _blobList.add(data);
    } else if (data is String) {
      JsonObject json = _decoder.convert(data);
      _ensureExistence(json.type);
      _router[json.type].add(json.data);
      _blobList.clear();
    }
  }
  
  _doneListener() {
    for (String key in _router) {
      _router[key].close();
    }
  }
  
  dynamic _revive(key, value) {
    if (key == null) {
      return value;
    } else {
      if (value is String) {
        if (value[0] == ':') {
          return value.substring(1);
        } else {
          int index = int.parse(value.substring(2));
          return _blobList[index];
        }
      } else {
        return value;
      }
    }
  }
  
  SocketUser(WebSocket this._socket, [String this._name]) {
     this._router = new Map<String, StreamEnhancer<JsonObject>>();
     this._blobList = new List<List<int>>();
     this._decoder = new JsonDecoder(_convert);
     this._socket
     .listen(_socketListener)
     .onDone(_doneListener)
     ;
  }
  
  set name(String value) {
    if (_name == null) {
      _name = value;
    } else {
      throw new Error();
    }
  }
  
  get name {
    return _name;
  }
}

==> 0.0.4.tar.gz: lib/src/socket_user.dart <==
library super_sockets.socket_user;

import 'dart:io';
import 'dart:async';
import 'dart:convert';
import 'dart:collection';
import 'package:json_object/json_object.dart';
import 'stream_enhancer.dart';

class SocketUser {
  String _name;
  WebSocket _socket;
  Map<String, WriteableStreamEnhancer<JsonObject>> _router;
  StreamController<JsonObject> _jsonCtrl;
  Function _convert;
  List<List<int>> _blobList;
  JsonDecoder _decoder;
    
  Stream<JsonObject> messageStream(String type) {
    _ensureExistence(type);
    return _router[type].stream;
  }
  
  Future<JsonObject> receive(String type) {
    return _jsonCtrl.stream.firstWhere((JsonObject jo) {
      return jo.type == type;
    });
  }
  
  Future<JsonObject> receiveAny(Iterable<String> types) {
    var matches = new Set<String>.from(types);
    
    return _jsonCtrl.stream.firstWhere((JsonObject jo) {
      return matches.contains(jo.type);
    });
  }
  
  void send(String type, JsonObject data){
    var sendable = new JsonObject();
    var blobQueue = new Queue<List<int>>();
    var fn = (value) {
      if (value is List<int>) {
        blobQueue.add(value);
        return "p:${blobQueue.length - 1}";
      } else if (value is String) {
        return ":$value";
      } else {
        return value;
      }
    };
    var jec = new JsonEncoder(fn);

    sendable.type = type;
    sendable.data = data;
    
    String jstring = jec.convert(sendable);
    for (List<int> blob in blobQueue) {
      _socket.add(blob);
    }
    
    _socket.add(jstring);
  }
  
  _ensureExistence(String type) {
    if (!_router.containsKey(type)) {
      _router[type] = new WriteableStreamEnhancer<JsonObject>.fromScratch();
    }
  }
    
  _socketListener(dynamic data) {
    if (data is List<int>) {
      _blobList.add(data);
    } else if (data is String) {
      JsonObject json = _decoder.convert(data);
      json.name = _name;
      _ensureExistence(json.type);
      _router[json.type].add(json);
      _blobList.clear();
    }
  }
  
  _doneListener() {
    for (String key in _router) {
      _router[key].close();
    }
  }
  
  _route(JsonObject jo) {
    _ensureExistence(jo.type);
    _router[jo.type].add(jo);
  }
  
  dynamic _revive(key, value) {
    if (key == null) {
      return value;
    } else {
      if (value is String) {
        if (value[0] == ':') {
          return value.substring(1);
        } else {
          int index = int.parse(value.substring(2));
          return _blobList[index];
        }
      } else {
        return value;
      }
    }
  }
  
  SocketUser(WebSocket this._socket, [String this._name]) {
     this._router = new Map<String, StreamEnhancer<JsonObject>>();
     this._jsonCtrl = new StreamController<JsonObject>();
     this._blobList = new List<List<int>>();
     this._decoder = new JsonDecoder(_convert);
     this._socket
     .listen(_socketListener)
     .onDone(_doneListener)
     ;  
     this._jsonCtrl.stream.listen(_route);
  }
  
  set name(String value) {
    if (_name == null) {
      _name = value;
    } else {
      throw new Error();
    }
  }
  
  get name {
    return _name;
  }
}
